#ifndef __DIB_H__
#define __DIB_H__

//---------------------------------------------------------------------------
//
//	File:			DIB.H
//	Programmer: 	Bob Provencher
//	Create Date:	11/9/94
//	Last Mod Date:	4/26/96
//	Description:	Dib class, formerly BitmapInfo & DIBitmap
//
//	Copyright (c) 1994, 1995. Aesir Software, Inc.
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef __DEBUG_H__
#include "Debug.h"
#endif

#define BITS_PER_BYTE	8		// silly, I know, but what the heck

class Dib
{

// attributes

private:

	LPBITMAPINFO	lpBmi;

// construction/destruction

public:

	Dib() :
		lpBmi( 0 )
	{
	}

	Dib( const string& filename ) :
		lpBmi( 0 )
	{
		Load( filename );
	}

	Dib( UINT id ) :
		lpBmi( 0 )
	{
		Load( id );
	}

	~Dib()
	{
		Cleanup();
	}

private:

	//
	//	if colors and bits are 0, lp MUST be a packed DIB
	//
	void Init( LPBITMAPINFO lp, RGBQUAD* colors = 0, LPBYTE bits = 0 );

	void Cleanup();

// attributes

public:

	LPBITMAPINFO GetBitmapInfo() const
	{
		ASSERT_VALID( this );
		return lpBmi;
	}

	//
	//	these functions optionally operate on a passed in value
	//	so I can compute sizes from external structures
	//

	LPBITMAPINFOHEADER GetBitmapInfoHeader( LPBITMAPINFO lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfo();
		ASSERT( lp != 0 );
		return (LPBITMAPINFOHEADER)lp;
	}

	DWORD GetSize( LPBITMAPINFOHEADER lp = 0 ) const		// should always return sizeof( BITMAPINFOHEADER )
	{
		if ( !lp )
			lp = GetBitmapInfoHeader();
		ASSERT( lp != 0 );
		return lp->biSize;
	}

	LONG GetWidth( LPBITMAPINFOHEADER lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfoHeader();
		ASSERT( lp != 0 );
		return lp->biWidth;
	}

	LONG GetHeight( LPBITMAPINFOHEADER lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfoHeader();
		ASSERT( lp != 0 );
		return lp->biHeight;
	}

	WORD GetBitCount( LPBITMAPINFOHEADER lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfoHeader();
		ASSERT( lp != 0 );
		return lp->biBitCount;
	}

	//
	//	for later use to support 15, 16 and 32 bit bitmaps, the bitcount
	//	will be different than those actually used for each pixel
	//

	WORD GetBitsPerPixel( LPBITMAPINFOHEADER lp = 0 ) const
	{
		WORD w = GetBitCount( lp );
		return w == ((WORD)15) ? ((WORD)16) : w;
	}

	DWORD GetCompression( LPBITMAPINFOHEADER lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfoHeader();
		ASSERT( lp != 0 );
		return lp->biCompression;
	}

	DWORD GetSizeImage( LPBITMAPINFOHEADER lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfoHeader();
		ASSERT( lp != 0 );
		return lp->biSizeImage;
	}

	DWORD GetNumColors( LPBITMAPINFOHEADER lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfoHeader();
		ASSERT( lp != 0 );
		DWORD biClrUsed = lp->biClrUsed;
		return biClrUsed ? biClrUsed : 1UL << GetBitCount( lp );
	}

	BOOL HasColorTable( LPBITMAPINFOHEADER lp = 0 ) const
	{
		return GetColorTableSize( lp ) > 0UL;
	}

	DWORD GetColorTableSize( LPBITMAPINFOHEADER lp = 0 ) const
	{
		return GetBitCount( lp ) < ((WORD)24) ? GetNumColors( lp ) : 0;
	}

	DWORD GetColorTableSizeBytes( LPBITMAPINFOHEADER lp = 0 ) const
	{
		return GetColorTableSize( lp ) * sizeof( RGBQUAD );
	}

	RGBQUAD* GetColorTable( LPBITMAPINFO lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfo();
		ASSERT( lp != 0 );
		return lp->bmiColors;
	}

    RGBQUAD Color( int index, LPBITMAPINFO lp = 0 ) const
    {
    	if ( !lp )
        	lp = GetBitmapInfo();
        ASSERT( lp != 0 );
        ASSERT( index < (int)GetColorTableSize( GetBitmapInfoHeader( lp ) ) );
        return lp->bmiColors[ index ];
    }

    RGBQUAD& Color( int index, LPBITMAPINFO lp = 0 )
    {
    	if ( !lp )
        	lp = GetBitmapInfo();
        ASSERT( lp != 0 );
        ASSERT( index < (int)GetColorTableSize( GetBitmapInfoHeader( lp ) ) );
        return lp->bmiColors[ index ];
    }

	DWORD GetPitch( LPBITMAPINFOHEADER lp = 0 ) const;		// width of scan line in bytes
	LPBYTE GetBitmap( LPBITMAPINFOHEADER lp = 0 ) const;	// pointer to the bitmap

	DWORD GetImageOffset( LPBITMAPINFOHEADER lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfoHeader();
		ASSERT( lp != 0 );
		return GetSize( lp ) + GetColorTableSizeBytes( lp );
	}

	DWORD GetImageBytes( LPBITMAPINFOHEADER lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfoHeader();
		ASSERT( lp != 0 );
		DWORD size = GetSizeImage( lp );
		if ( size )
			return size;
		else
			return GetPitch( lp ) * GetHeight( lp );
	}

	DWORD GetPackedDibSize( LPBITMAPINFOHEADER lp = 0 ) const
	{
		if ( !lp )
			lp = GetBitmapInfoHeader();
		ASSERT( lp != 0 );
		return GetImageOffset( lp ) + GetImageBytes( lp );
	}

	void FindColor( COLORREF color, BYTE& byte ) const;
	void UpdateColors( COLORREF color, COLORREF new_color );

// methods

	//
	//	load from file
	//

	void Load( const string& filename );
	void Load( ifstream& in );

	//
	//	load from resource file
	//

	void Load( UINT id );

	//
	//	conversion routines
	//

	// GetDIBits : convert HBITMAP to DIB
	void GetBits( HBITMAP hBmp, HPALETTE& hPal );

	// SetDIBits : convert DIB to HBITMAP (put bits into DDB format)
	void SetBits( HBITMAP& hBmp ) const;
	void SetMaskBits( HBITMAP& hBmp, COLORREF trans ) const;

	// CreateDIBitmap : put bits into new bitmap
	void CreateBitmap( HBITMAP& hBmp ) const;
	void CreateMaskBitmap( HBITMAP& hBmp, COLORREF trans ) const;

	// SetDIBitsToDevice : put to screen
	void SetBitsToDevice( HDC hDC, const POINT& ptDest,  const RECT& rcSrc ) const;

	// CreateDIBSection: get an HBITMAP to a DIB for drawing and blitting
	void CreateSection( HBITMAP& hBmp ) const;

public:

#ifdef _DEBUG
	void AssertValid() const;
	void Dump( ostream& debug ) const;
#endif

};

#endif
