//---------------------------------------------------------------------------
//
//	File:			LOGPAL.CPP
//	Programmer: 	Bob Provencher
//	Create Date:	11/9/94
//	Last Mod Date:	4/26/96
//	Description:	LogicalPalette class implementation
//
//	Copyright (c) 1994, 1995. Aesir Software, Inc.
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#include "SysInc.h"
#pragma hdrstop

#ifndef __COMMON_H__
#include "Common.h"
#endif

#ifndef __LOGPAL_H__
#include "LogPal.h"
#endif

#ifndef __DIB_H__
#include "Dib.h"
#endif

LogPalette::LogPalette( const RGBQUAD& color, WORD num, BYTE flags ) :
	lpLogPalette( 0 )
{
	ResizeTo( num );
	for ( int i = 0; i < num; i++ )
		Copy( Entry( i ), color );
	SetEntryFlags( 0, num, flags );
}

void LogPalette::ResizeTo( WORD wNewSize )
{
	if ( IsCreated() )
    	ASSERT_VALID( this );
	DWORD dwNewBytes = NumBytes( wNewSize );
	DWORD dwBytes = NumBytes();
	LOGPALETTE FAR* lpNewLogPalette = 0;
	if ( dwNewBytes )
	{
		lpNewLogPalette = (LOGPALETTE FAR*)new LPBYTE[ (size_t)dwNewBytes ];
		memset( lpNewLogPalette, 0, (size_t)dwNewBytes );
	}
	if ( lpNewLogPalette )
	{
		if ( lpLogPalette )
		{
			memcpy( lpNewLogPalette, lpLogPalette,
					  (size_t)min( dwBytes, dwNewBytes ) );
		}
		else
			lpNewLogPalette->palVersion = 0x0300;
		lpNewLogPalette->palNumEntries = wNewSize;
	}
	delete [] (LPBYTE)lpLogPalette;
	lpLogPalette = lpNewLogPalette;
}

BOOL LogPalette::GetSystemEntries( UINT uStart, UINT uEntries )
{
	ASSERT_VALID( this );
	ASSERT( uStart + uEntries <= NumEntries() )
	HDC hDC = GetDC( 0 );
	if ( hDC )
	{
		UINT u = GetSystemPaletteEntries( hDC, uStart, uEntries,
										  lpLogPalette->palPalEntry +
										  	uStart );
		ReleaseDC( 0, hDC );
		return u == uEntries;
	}
	return FALSE;
}

void LogPalette::CopyEntries( const Dib& dib )
{
	int colors = dib.GetNumColors();
	ResizeTo( (WORD)colors );
	for ( int i = 0; i < colors; i++ )
		Copy( Entry( i ), dib.Color( i ) );
}

void LogPalette::SetEntryFlags( WORD wStart, WORD wEntries, BYTE peFlags )
{
	ASSERT_VALID( this );
	ASSERT( wStart + wEntries <= NumEntries() );
	for ( int i = wStart; i < wStart + wEntries; i++ )
		Entry( i ).peFlags = peFlags;
}

BOOL LogPalette::EasyStepTo( const LogPalette& pal )
{
	ASSERT_VALID( this );
	ASSERT_VALID( &pal );
    BOOL fReturn = FALSE;
	for ( int i = 0; !fReturn && i < NumEntries(); i++ )
	{
		PALETTEENTRY& entry = Entry( i );
		PALETTEENTRY  tgt   = pal[ i ];
		fReturn = StepTo( entry.peRed,   tgt.peRed )   && fReturn;
		fReturn = StepTo( entry.peGreen, tgt.peGreen ) && fReturn;
		fReturn = StepTo( entry.peBlue,  tgt.peBlue )  && fReturn;
		entry.peFlags = tgt.peFlags;
	}
    return fReturn;
}

void LogPalette::EvenStepTo( BYTE start, BYTE& tgt, float step, int stepno )
{
	tgt = (BYTE)( (float)start + ( step * (float)stepno ) );
}

void LogPalette::EvenStepTo( const PALETTEENTRY& start, PALETTEENTRY& tgt,
							 const PaletteStep& step, int stepno )
{
	EvenStepTo( start.peRed,   tgt.peRed,   step.stRed,   stepno );
	EvenStepTo( start.peGreen, tgt.peGreen, step.stGreen, stepno );
	EvenStepTo( start.peBlue,  tgt.peBlue,  step.stBlue,  stepno );
}

void LogPalette::EvenStepTo( const LogPalette& start, PaletteStep* step, 
							 int stepno )
{
	for ( int i = 0; i < NumEntries(); i++ )
		EvenStepTo( start.Entry( i ), Entry( i ), step[ i ], stepno );
}

void LogPalette::Negate()
{
	ASSERT_VALID( this );
	for ( int i = 0; i < NumEntries(); i++ )
	{
		PALETTEENTRY& pal = Entry( i );
		pal.peRed   = ((BYTE)~pal.peRed);
		pal.peGreen = ((BYTE)~pal.peGreen);
		pal.peBlue  = ((BYTE)~pal.peBlue);
	}
}

LogPalette LogPalette::operator ~() const
{
	LogPalette pal( *this );
	pal.Negate();
	return pal;
}

void LogPalette::ComputeFadeStep( BYTE a, BYTE b, float& step, BYTE steps )
{
	step = ( (float)b - (float)a ) / (float)steps;
}

void LogPalette::ComputeFadeStep( const PALETTEENTRY& start, 
							      const PALETTEENTRY& end, 
							      PaletteStep& step, BYTE steps )
{
	ComputeFadeStep( start.peRed,   end.peRed,   step.stRed,   steps );
	ComputeFadeStep( start.peGreen, end.peGreen, step.stGreen, steps );
	ComputeFadeStep( start.peBlue,  end.peBlue,  step.stBlue,  steps );
}

void LogPalette::ComputeFadeSteps( const LogPalette& target, 
								   PaletteStep* step, 
								   BYTE steps )
{
	for ( int i = 0; i < NumEntries(); i++ )
		ComputeFadeStep( Entry( i ), target.Entry( i ), step[ i ], steps );
}

#ifdef _DEBUG
void LogPalette::AssertValid() const
{
	ASSERT( lpLogPalette != 0 );
}

void LogPalette::Dump( ostream& debug ) const
{
	debug << "Start LogPalette"				   << endl
		  << " this: "         << (void*)this  << endl
		  << " lpLogPalette: " << lpLogPalette << endl;
	if ( IsCreated() )
		Dump( debug, lpLogPalette );
	debug << "End LogPalette"				   << endl;
}

void LogPalette::Dump( ostream& debug, const LOGPALETTE FAR* pal ) const
{
	debug << " Start LOGPALETTE"                       << endl
		  << "  palVersion:    " << pal->palVersion    << endl
		  << "  palNumEntries: " << pal->palNumEntries << endl;
	DWORD c = NumEntries( pal );
	if ( c > 0 )
	{
		debug << "  Entries:" << endl;
		for ( UINT i = 0; i < c; i++ )
		{
			debug << "   palPalEntry[ " << (BYTE)i << " ]: ";
			Dump( debug, Entry( i, pal ) );
		}
	}
	debug << " End LOGPALETTE"                        << endl;
}

void LogPalette::Dump( ostream& debug, const PALETTEENTRY& pal ) const
{
	debug << " peRed: "   << pal.peRed
		  << " peGreen: " << pal.peGreen
		  << " peBlue: "  << pal.peBlue
		  << " peFlags: " << pal.peFlags << endl;
}
#endif

