#ifndef __LOGPAL_H__
#define __LOGPAL_H__

//---------------------------------------------------------------------------
//
//	File:			LOGPAL.H
//	Programmer: 	Bob Provencher
//	Create Date:	11/9/94
//	Last Mod Date:	4/26/96
//	Description:	LogPalette class definition
//
//	Copyright (c) 1994, 1995. Aesir Software, Inc.
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef __SYSINC_H__
#include "SysInc.h"
#endif

#ifndef __CLASSDEF_H__
#include "ClassDef.h"
#endif

class LogPalette
{

private:

	LOGPALETTE FAR*	lpLogPalette;

	UINT NumBytes( WORD wPalSize ) const
	{
		return wPalSize > 0
			? ( 2 * sizeof( WORD ) ) + ( wPalSize * sizeof( PALETTEENTRY ) )
			: 0;
	}

	BOOL StepTo( BYTE& b, BYTE target )
	{
		if ( b < target )
			b++;
		else if ( b > target )
			b--;
		return b == target;
	}

// Construction/Destruction

public:

	LogPalette() :
		lpLogPalette( 0 )
	{
    	ResizeTo( 256 );
        ASSERT_VALID( this );
	}

	LogPalette( const LogPalette& pal ) :
		lpLogPalette( 0 )
	{
		Copy( pal );
		ASSERT_VALID( this );
	}

	LogPalette( const RGBQUAD& color, WORD num = 256, BYTE flags = 0 );

	LogPalette( const Dib& dib ) :
    	lpLogPalette( 0 )
	{
		CopyEntries( dib );
		ASSERT_VALID( this );
	}

	~LogPalette()
	{
		ASSERT_VALID( this );
		if ( IsCreated() )
			Delete();
	}

// Query attributes

	LOGPALETTE FAR* Palette() const
	{
		ASSERT_VALID( this );
		return lpLogPalette;
	}

	BOOL IsCreated() const
	{
		return (BOOL)lpLogPalette;
	}

	WORD NumEntries( const LOGPALETTE FAR* pal = 0 ) const
	{
		if ( !pal )
			pal = lpLogPalette;
		return pal ? pal->palNumEntries : ((WORD)0);
	}

	PALETTEENTRY& Entry( UINT index, LOGPALETTE FAR* pal = 0 )
	{
		if ( !pal )
			pal = lpLogPalette;
		ASSERT( pal != 0 );
		ASSERT( index < NumEntries( pal ) );
		return pal->palPalEntry[ index ];
	}

	PALETTEENTRY Entry( UINT index, const LOGPALETTE FAR* pal = 0 ) const
	{
		if ( !pal )
			pal = lpLogPalette;
		ASSERT( pal != 0 );
		ASSERT( index < NumEntries( pal ) )
        return pal->palPalEntry[ index ];
	}

	PALETTEENTRY FAR* Entries( LOGPALETTE FAR* pal = 0  ) const
	{
		if ( !pal ) 
			pal = lpLogPalette;
		ASSERT( pal != 0 );
    	return pal->palPalEntry;
	}

	PALETTEENTRY& operator[] ( UINT index )
	{
		ASSERT_VALID( this );
		return Entry( index );
	}

	PALETTEENTRY operator[] ( UINT index ) const
	{
		ASSERT_VALID( this );
		return Entry( index );
	}

	LogPalette& operator = ( const LogPalette& pal )
	{
		Copy( pal );
		return *this;
	}

// methods

	void Copy( const LogPalette& pal )
	{
		ResizeTo( pal.NumEntries() );
		if ( lpLogPalette )
			memcpy( lpLogPalette, pal.lpLogPalette, (size_t)NumBytes() );
	}

	void Copy( PALETTEENTRY& pal, const RGBQUAD& rgb )
	{
		pal.peRed   = rgb.rgbRed;
		pal.peGreen = rgb.rgbGreen;
		pal.peBlue  = rgb.rgbBlue;
	}

	void CopyEntries( const Dib& bi );

	void Delete()
	{
		ASSERT_VALID( this );
		ResizeTo( 0 );
	}

	UINT NumBytes() const
	{
		return NumBytes( NumEntries() );
	}

	void ResizeTo( WORD wNewSize );

	BOOL GetSystemEntries( UINT uStart, UINT uEntries );

	void SetEntryFlags( WORD wStart, WORD wEntries, BYTE peFlags );

	BOOL EasyStepTo( const LogPalette& pal );

	void EvenStepTo( BYTE start, BYTE& tgt, float step, int stepno );
	void EvenStepTo( const PALETTEENTRY& start, PALETTEENTRY& tgt, 
 					 const PaletteStep& step, int stepno );
	void EvenStepTo( const LogPalette& start, PaletteStep* step, 
					 int stepno );

	void Negate();

	LogPalette operator ~() const;

	void ComputeFadeStep( BYTE a, BYTE b, float& step, BYTE steps );
	void ComputeFadeStep( const PALETTEENTRY& start,
						  const PALETTEENTRY& end,
						  PaletteStep& step, BYTE steps );
	void ComputeFadeSteps( const LogPalette& tgt, PaletteStep* step,
						   BYTE steps );

	#ifdef _DEBUG

	void AssertValid() const;

	void Dump( ostream& debug ) const;
	void Dump( ostream& debug, const LOGPALETTE FAR* pal ) const;
	void Dump( ostream& debug, const PALETTEENTRY& entry ) const;

	ostream& operator << ( ostream& debug ) const
	{
		ASSERT_VALID( this );
		Dump( debug );
        return debug;
	}

	#endif

};

struct PaletteStep
{
	float stRed;
	float stGreen;
	float stBlue;
};

#endif





